
#include <iostream>
#include "ros/ros.h"
#include "std_msgs/String.h"

#include <sstream>
#include <rbdl/rbdl.h>

#ifndef RBDL_BUILD_ADDON_URDFREADER
	#error "Error: RBDL addon URDFReader not enabled."
#endif

#include <rbdl/addons/urdfreader/urdfreader.h>

using namespace RigidBodyDynamics;
using namespace RigidBodyDynamics::Math;

int main (int argc, char* argv[]) {
	ros::init(argc, argv, "rbdl_test");
        ros::NodeHandle n;
	rbdl_check_api_version (RBDL_API_VERSION);

	Model* model = new Model();

	if (!Addons::URDFReadFromFile ("./samplemodel.urdf", model, false)) {
		std::cerr << "Error loading model ./samplemodel.urdf" << std::endl;
		abort();
	}


//	if (argc != 2) {

//	std::cerr << "Error: not right number of arguments." << std::endl;

//	std::cerr << "usage: " << argv[0] << " <n.urdf>" << std::endl;

//	exit(-1);

//	  }



//	if (!Addons::URDFReadFromFile (argv[1], model, false)) {

//	  std::cerr << "Error loading model " << argv[1] << std::endl;

//	  abort();

//	  }


//	  std::cout << "Degree of freedom overview:" << std::endl;
//	  std::cout << Utils::GetModelDOFOverview(*model);

//	  std::cout << "Model Hierarchy:" << std::endl;
//	  std::cout << Utils::GetModelHierarchy(*model);

//	  VectorNd Q = VectorNd::Zero (model->q_size);
//	  VectorNd QDot = VectorNd::Zero (model->qdot_size);
//	  VectorNd Tau = VectorNd::Zero (model->qdot_size);
//	  VectorNd QDDot = VectorNd::Zero (model->qdot_size);

//	  std::cout << "Forward Dynamics with q, qdot, tau set to zero:" << std::endl;
//	  ForwardDynamics (*model, Q, QDot, Tau, QDDot);

//	  std::cout << QDDot.transpose() << std::endl;

/////////////////////////////////////////////////////////////////


//	  Model* model = NULL;
//	  unsigned int body_a_id, body_b_id, body_c_id;
//	  Body body_a, body_b, body_c;
//	  Joint joint_a, joint_b, joint_c;
//	  model = new Model();
//	  model->gravity = Vector3d (0., -9.81, 0.);
//	  body_a = Body (1., Vector3d (0.5, 0., 0.0), Vector3d (1., 1., 1.));
//	    joint_a = Joint(
//	    JointTypeRevolute,
//	    Vector3d (0., 0., 1.)
//	  );
//	  
//	  body_a_id = model->AddBody(0, Xtrans(Vector3d(0., 0., 0.)), joint_a, body_a);
//	  
//	  body_b = Body (1., Vector3d (0., 0.5, 0.), Vector3d (1., 1., 1.));
//	    joint_b = Joint (
//	    JointTypeRevolute,
//	    Vector3d (0., 0., 1.)
//	  );
//	  
//	  body_b_id = model->AddBody(body_a_id, Xtrans(Vector3d(1., 0., 0.)), joint_b, body_b);
//	  
//	  body_c = Body (0., Vector3d (0.5, 0., 0.), Vector3d (1., 1., 1.));
//	    joint_c = Joint (
//	    JointTypeRevolute,
//	    Vector3d (0., 0., 1.)
//	  );
//	  
//	  body_c_id = model->AddBody(body_b_id, Xtrans(Vector3d(0., 1., 0.)), joint_c, body_c);


	  std::cout << "Degree of freedom overview:" << std::endl;
	  std::cout << Utils::GetModelDOFOverview(*model);

	  std::cout << "Model Hierarchy:" << std::endl;
	  std::cout << Utils::GetModelHierarchy(*model);

	  VectorNd Q = VectorNd::Zero (model->q_size);
	  VectorNd QDot = VectorNd::Zero (model->qdot_size);
	  VectorNd Tau = VectorNd::Zero (model->qdot_size);
	  VectorNd QDDot = VectorNd::Zero (model->qdot_size);

	  std::cout << "Forward Dynamics with q, qdot, tau set to zero:" << std::endl;
	  ForwardDynamics (*model, Q, QDot, Tau, QDDot);

	  std::cout << QDDot.transpose() << std::endl;
//	VectorNd Q = VectorNd::Zero (model->dof_count);
//	VectorNd QDot = VectorNd::Zero (model->dof_count);
//	VectorNd Tau = VectorNd::Zero (model->dof_count);
//	VectorNd QDDot = VectorNd::Zero (model->dof_count);

// 	ForwardDynamics (*model, Q, QDot, Tau, QDDot);

//	std::cout << Q.transpose() << std::endl;
//	std::cout << QDDot.transpose() << std::endl;

	delete model;

 	return 0;
}

