#include <rbdl/addons/urdfreader/urdfreader.h>
#include <rbdl/rbdl_utils.h>
#include <rbdl/rbdl.h>

int main(int argc, char** argv){
    
    RigidBodyDynamics::Model model;
    RigidBodyDynamics::Addons::URDFReadFromFile("../urdf/nao.urdf", &model, false, false);
    
    Eigen::VectorXd q, qdot, qddot;
    q.setRandom(model.q_size);
    qdot.setRandom(model.qdot_size);
    qddot.setRandom(model.qdot_size);
    
    /* Update kinematics with qdot != 0 */
    RigidBodyDynamics::UpdateKinematics(model, q, qdot, qddot);
    
    /* Fill B with the inertia */
    Eigen::MatrixXd B;
    B.setZero(model.qdot_size, model.qdot_size);
    RigidBodyDynamics::CompositeRigidBodyAlgorithm(model, q, B);

    /* Get a random vector x */
    Eigen::VectorXd x;
    x.setRandom(model.qdot_size);
    
    /* Compute Binv * x via Eigen */
    Eigen::VectorXd Binv_x = B.inverse() * x;
    
    /* Compute Binv * x via RBDL */
    Eigen::VectorXd Binv_x_rbdl(model.qdot_size);
    RigidBodyDynamics::CalcMInvTimesTau(model, q, x, Binv_x_rbdl, true);
    
    /* The error norm is far from zero */
    std::cout << "Large error here!: " << (Binv_x-Binv_x_rbdl).norm() << std::endl;
    
    /* Update kinematics with qdot = 0 */
    RigidBodyDynamics::UpdateKinematics(model, q, qdot*0, qddot);
    
    /* Compute Binv * x via RBDL */
    RigidBodyDynamics::CalcMInvTimesTau(model, q, x, Binv_x_rbdl, true);
    
    /* The error norm is zero zero */
    std::cout << "With qdot = 0 very small error: " << (Binv_x-Binv_x_rbdl).norm() << std::endl;
    
    
}